"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var idSequence = 0;
var Entity = /** @class */ (function () {
    function Entity(_registerChange) {
        this._components = Object.create(null);
        this._changeRegistered = false;
        this.id = idSequence++;
        this._registerChange = _registerChange;
    }
    Entity.prototype.add = function (instance) {
        if (instance == null) {
            throw new Error('Entity.add :: Argument is not a component instance.');
        }
        var componentClass = instance.constructor;
        if (!componentClass || !componentClass.id) {
            throw new Error('Entity.add :: Argument is not a component instance.');
        }
        else if (this._components[componentClass.id]) {
            throw new Error('Entity.add :: Component class already present.');
        }
        this._components[componentClass.id] = instance;
        this._onChange();
        return this;
    };
    Entity.prototype.has = function (componentClass) {
        if (!componentClass || !componentClass.id) {
            throw new Error('Entity.has :: Argument is not a component class.');
        }
        return !!this._components[componentClass.id];
    };
    Entity.prototype.get = function (componentClass) {
        if (!componentClass || !componentClass.id) {
            throw new Error('Entity.get :: Argument is not a component class.');
        }
        var component = this._components[componentClass.id];
        if (!component) {
            throw new Error('Entity.get :: Component class not present.');
        }
        return component;
    };
    Entity.prototype.remove = function (componentClass) {
        if (!componentClass || !componentClass.id) {
            throw new Error('Entity.remove :: Argument is not a component class.');
        }
        this._components[componentClass.id] = undefined;
        this._onChange();
        return this;
    };
    Entity.prototype._onChange = function () {
        if (!this._changeRegistered && this._registerChange) {
            this._registerChange(this);
            this._changeRegistered = true;
        }
    };
    return Entity;
}());
exports.Entity = Entity;
function notifyAfterChangeRegistered(entity) {
    entity._changeRegistered = false;
}
exports.notifyAfterChangeRegistered = notifyAfterChangeRegistered;
//# sourceMappingURL=Entity.js.map