"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Entity_1 = require("./Entity");
var Events_1 = require("./Events");
var GameWorld = /** @class */ (function () {
    function GameWorld(_queries) {
        var _this = this;
        this._changedEntities = [];
        this._removedEntities = [];
        this._events = new Events_1.Events();
        this._onEntityChange = function (entity) { return _this._changedEntities.push(entity); };
        this._queries = _queries;
    }
    Object.defineProperty(GameWorld.prototype, "time", {
        get: function () {
            return this._time || 0;
        },
        enumerable: true,
        configurable: true
    });
    GameWorld.prototype.createEntity = function (assemblage) {
        var entity = new Entity_1.Entity(this._onEntityChange);
        if (assemblage) {
            assemblage(entity);
        }
        return entity;
    };
    GameWorld.prototype.removeEntity = function (entity) {
        this._removedEntities.push(entity);
    };
    GameWorld.prototype.emit = function (event) {
        this._events.emit(event, this.time);
    };
    GameWorld.prototype._internal_nextEvent = function () {
        return this._events.get();
    };
    GameWorld.prototype._internal_tick = function (time) {
        this._time = time;
        this.emit('tick');
    };
    GameWorld.prototype._internal_handleChanges = function () {
        for (var _i = 0, _a = this._changedEntities; _i < _a.length; _i++) {
            var entity = _a[_i];
            for (var _b = 0, _c = this._queries; _b < _c.length; _b++) {
                var query = _c[_b];
                query.onChange(entity);
            }
            Entity_1.notifyAfterChangeRegistered(entity);
        }
        this._changedEntities.length = 0;
        for (var _d = 0, _e = this._removedEntities; _d < _e.length; _d++) {
            var entity = _e[_d];
            for (var _f = 0, _g = this._queries; _f < _g.length; _f++) {
                var query = _g[_f];
                query.onRemove(entity);
            }
        }
        this._removedEntities.length = 0;
    };
    return GameWorld;
}());
exports.GameWorld = GameWorld;
//# sourceMappingURL=World.js.map