import { GameWorld } from './World';
import { toInternalSystem } from './systems';
var Game = /** @class */ (function () {
    function Game(systems, init) {
        var _this = this;
        this._systems = {};
        this._subscriptions = [];
        var queries = [];
        systems
            .map(toInternalSystem)
            .forEach(function (system) {
            _this._systems[system.on] = _this._systems[system.on] || [];
            _this._systems[system.on].push(system);
            if (system.query) {
                queries.push(system.query);
            }
        });
        this._world = new GameWorld(queries);
        if (init) {
            init(this._world);
        }
    }
    Game.prototype.update = function (time) {
        this._world._internal_tick(time);
        while (true) {
            var event_1 = this._world._internal_nextEvent();
            if (!event_1) {
                break;
            }
            for (var _i = 0, _a = this._subscriptions; _i < _a.length; _i++) {
                var listener = _a[_i];
                listener(event_1);
            }
            if (this._systems[event_1.type]) {
                for (var _b = 0, _c = this._systems[event_1.type]; _b < _c.length; _b++) {
                    var system = _c[_b];
                    this._world._internal_handleChanges();
                    system.process(system.query ? system.query.entities : [], this._world, event_1);
                }
            }
        }
    };
    Game.prototype.start = function () {
        var _this = this;
        var update = function () {
            requestAnimationFrame(update);
            _this.update(Date.now() / 1000);
        };
        update();
    };
    Game.prototype.subscribe = function (listener) {
        var _this = this;
        this._subscriptions.push(listener);
        return function () {
            _this._subscriptions.splice(_this._subscriptions.indexOf(listener), 1);
        };
    };
    return Game;
}());
export { Game };
//# sourceMappingURL=Game.js.map