export function createQuery(selectors) {
    if (Array.isArray(selectors)) {
        return new MultiQuery(selectors);
    }
    else {
        return new SingleQuery(selectors);
    }
}
var SingleQuery = /** @class */ (function () {
    function SingleQuery(_selector) {
        this._entities = [];
        this._indexMap = {};
        if (typeof _selector !== 'function') {
            throw new Error('new Query :: selector must be a function');
        }
        this._selector = _selector;
    }
    Object.defineProperty(SingleQuery.prototype, "entities", {
        get: function () {
            return this._entities;
        },
        enumerable: true,
        configurable: true
    });
    SingleQuery.prototype.onChange = function (entity) {
        if (this._selector(entity)) {
            if (this._indexMap[entity.id] == null) {
                this._indexMap[entity.id] = this.entities.length;
                this._entities.push(entity);
            }
        }
        else {
            this.onRemove(entity);
        }
    };
    SingleQuery.prototype.onRemove = function (entity) {
        var index = this._indexMap[entity.id];
        if (index != null) {
            var last = this._entities.pop();
            if (last !== entity) {
                this._entities[index] = last;
                this._indexMap[last.id] = index;
            }
        }
    };
    return SingleQuery;
}());
export { SingleQuery };
var MultiQuery = /** @class */ (function () {
    function MultiQuery(_selectors) {
        this._queries = _selectors.map(function (selector) { return new SingleQuery(selector); });
    }
    Object.defineProperty(MultiQuery.prototype, "entities", {
        get: function () {
            return this._queries.map(function (query) { return query.entities; });
        },
        enumerable: true,
        configurable: true
    });
    MultiQuery.prototype.onChange = function (entity) {
        for (var _i = 0, _a = this._queries; _i < _a.length; _i++) {
            var query = _a[_i];
            query.onChange(entity);
        }
    };
    MultiQuery.prototype.onRemove = function (entity) {
        for (var _i = 0, _a = this._queries; _i < _a.length; _i++) {
            var query = _a[_i];
            query.onRemove(entity);
        }
    };
    return MultiQuery;
}());
export { MultiQuery };
//# sourceMappingURL=Query.js.map